Package: tmvmixnorm
Type: Package
Title: Sampling from Truncated Multivariate Normal and t Distributions
Version: 1.2.0
Authors@R: c(person("Ting Fung", "Ma", email = "tingfung@mailbox.sc.edu", role = c("cre", "aut")),
    person("Sujit K.", "Ghosh", email = "sujit_ghosh@ncsu.edu", role = "aut"),
    person("Yifang", "Li", role = "aut"))
Maintainer: Ting Fung Ma <tingfung@mailbox.sc.edu>
Description: Efficient sampling of truncated multivariate (scale) mixtures of normals under linear inequality constraints is nontrivial due to the analytically intractable normalizing constant. Meanwhile, traditional methods may subject to numerical issues, especially when the dimension is high and dependence is strong.    Algorithms proposed by Li and Ghosh (2015) <doi: 10.1080/15598608.2014.996690> are adopted for overcoming difficulties in simulating truncated distributions. Efficient rejection sampling for simulating truncated univariate normal distribution is included in the package, which shows superiority in terms of acceptance rate and numerical stability compared to existing methods and R packages. An efficient function for sampling from truncated multivariate normal distribution subject to convex polytope restriction regions based on Gibbs sampler for conditional truncated univariate distribution is provided. By extending the sampling method, a function for sampling truncated multivariate Student's t distribution is also developed.     Moreover, the proposed method and computation remain valid for high dimensional and strong dependence scenarios. Empirical results in Li and Ghosh (2015) <doi: 10.1080/15598608.2014.996690> illustrated the superior performance in terms of various criteria (e.g. mixing and integrated auto-correlation time).
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.2.0)
Imports: stats, MASS
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-08 13:19:57 UTC; ralph
Author: Ting Fung Ma [cre, aut],
  Sujit K. Ghosh [aut],
  Yifang Li [aut]
Repository: CRAN
Date/Publication: 2026-05-11 07:50:02 UTC
Built: R 4.5.2; ; 2026-05-11 10:39:11 UTC; unix

# Generated by roxygen2: do not edit by hand

export(dtuvn)
export(exp_acc_opt)
export(exp_rej)
export(halfnorm_acc)
export(halfnorm_rej)
export(imp)
export(imp_acc)
export(norm_acc)
export(norm_rej)
export(ptuvn)
export(rtmvn)
export(rtmvt)
export(rtuvn)
export(unif_acc)
export(unif_rej)

#' Density function of truncated univariate normal distribution
#'
#' \code{dtuvn} calculates the probability density function (pdf) of truncated univariate normal distribution.
#'
#' @param x value at which density is desired.
#' @param mean mean of the underlying univariate normal distribution.
#' @param sd standard deviation of the underlying univariate normal distribution.
#' @param lower lower bound for truncation.
#' @param upper upper bound for truncation.
#' @export
#' @return \code{dtuvn} returns the density (with same dimension and type as \code{x}) of truncated univariate normal distribution.
#'
#' @export
#' 
#' @examples
#' dtuvn(x= -3:3, mean=0, sd=1 ,lower= -2, upper=2)
#'
dtuvn <- function(x,mean,sd,lower,upper){
  value <- stats::dnorm(x,mean,sd)/(stats::pnorm(upper,mean,sd) - stats::pnorm(lower,mean,sd))
  return(ifelse(x<lower | x>upper,0,value))
}

#' Random number generation for truncated multivariate Student's t distribution subject to linear inequality constraints
#'
#' \code{rtmvt} simulates truncated multivariate (p-dimensional) Student's t distribution subject to linear inequality constraints. The constraints should be written as a matrix (\code{D}) with \code{lower} and \code{upper} as the lower and upper bounds for those constraints respectively. Note that \code{D} can be non-full rank, which generalizes many traditional methods.
#'
#' @param n number of random samples desired (sample size).
#' @param Mean location vector of the multivariate Student's t distribution.
#' @param Sigma positive definite dispersion matrix of the multivariate t distribution.
#' @param nu degrees of freedom for Student-t distribution.
#' @param D matrix or vector of coefficients of linear inequality constraints.
#' @param lower lower bound vector for truncation.
#' @param upper upper bound vector for truncation.
#' @param int initial value vector for Gibbs sampler (satisfying truncation), if \code{NULL} then determine automatically.
#' @param burn burn-in iterations discarded (default as \code{10}).
#' @param thin thinning lag (default as \code{1}).
#'
#' @export
#' 
#' @return \code{rtmvt} returns a (\code{n*p}) matrix (or vector when \code{n=1}) containing random numbers which follows truncated multivariate Student-t distribution.
#'
#' @examples
#' # Example for full rank
#' d <- 3
#' rho <- 0.5
#' nu <- 10
#' Sigma <- matrix(0, nrow=d, ncol=d)
#' Sigma <- rho^abs(row(Sigma) - col(Sigma))
#'
#' D1 <- diag(1,d) # Full rank
#'
#' set.seed(1203)
#' ans.t <- rtmvt(n=1000, Mean=1:d, Sigma, nu=nu, D=D1, lower=rep(-1,d), upper=rep(1,d),
#' burn=50, thin=0)
#'
#' apply(ans.t, 2, summary)
#'
rtmvt <- function(n, Mean, Sigma, nu, D, lower, upper, int=NULL, burn=10, thin=1){

  if ( any(lower >= upper)) stop("lower bound must be smaller than upper bound\n")
  bound.check <- 0

  if (!is.null(int)) {
    inits_test <- D%*%int
    lower.log <- inits_test >= lower + 1e-8  # small tol for get away from bound
    upper.log <- inits_test <= upper - 1e-8  # small tol for get away from bound
    bound.check <- prod(lower.log*upper.log)

  } else if (bound.check == 0) {

    D.inv <- MASS::ginv(D)
    int <- D.inv%*%(lower + upper)/2
  }

  if( any (c(burn,thin,n) %% 1 != 0))  stop("burn, thin and n must be integer\n")
  if ( any(c(burn, thin, n -1) < 0) ) stop("burn, thin must be  non-negative interger, n must be positive integer\n")


  if(is.vector(D)==TRUE){
    Rtilde <- t(as.matrix(D))
    lower <- as.vector(lower)
    upper <- as.vector(upper)
  } else {
    Rtilde <- D
  }

  a <- lower - Rtilde%*%Mean
  b <- upper - Rtilde%*%Mean
  Sigma.chol <- t(chol(Sigma))
  R <- Rtilde%*%Sigma.chol

  p <- ncol(R) # number of parameters, i.e. length of beta vector

  x <- solve(Sigma.chol)%*%(int-Mean) # initial value for the transformed tmvt
  # int is the initial value for the original problem

  keep.t <- matrix(NA, ncol=p, nrow=(thin+1)*n+burn)

  for (i in 1:((thin+1)*n+burn)){
    u <- stats::rchisq(1,df=nu) # sample from chisq(nu)
    denom <- sqrt(u/nu)
    lw <- c(a*denom)
    up <- c(b*denom)
    z0 <- c(x*denom)  # initial value for the tmvn for this step
    z <- c( rtmvn(n=1,Mean=rep(0,p),Sigma=diag(1,p),D=R,lower=lw,upper=up,int=z0, burn=0))
    # sample from standard tmvn

    x <- z/denom  # sample from standard tmvt

    ############################################################
    # for the original results
    w <- Sigma.chol%*%x+Mean
    keep.t[i,] <- w
  }

  final.ind <- 1:((thin+1)*n+burn)
  final.ind <- final.ind[(burn+1):length(final.ind)]
  final.ind <- seq(1,length(final.ind),by=thin+1) + thin + burn

  if (n == 1) {result <- c(keep.t[final.ind,])  } else{ result <- keep.t[final.ind,]}
  return( result )
}

#' Acceptance rate of translated-exponential rejection sampling
#'
#' \code{exp_acc_opt} calculates the acceptance rate of translated-exponential rejection sampling for the truncation interval (a,b).
#'
#' @param a lower bound for truncation.
#' @param b upper bound for truncation.
#' 
#' @export
#' 
#' @examples
#' 
#' set.seed(1203)
#' exp_acc_opt(1,2)
#'
exp_acc_opt <- function(a,b){
  lambda <- ( a+sqrt(a^2+4) )/2
  rate <- sqrt(2*pi)*(stats::pnorm(b)-stats::pnorm(a))*lambda*exp(-lambda^2/2+lambda*a)
  return(rate)
}

#' Acceptance rate of uniform rejection sampling
#'
#' \code{unif_acc} calculates the acceptance rate of uniform rejection sampling for the truncation interval (a,b).
#'
#' @param a lower bound for truncation.
#' @param b upper bound for truncation.
#'
#' @export
#' 
#' @examples
#' set.seed(1203)
#' unif_acc(1,2)
#'
unif_acc <- function(a,b){
  cons <- sqrt(2*pi)/(b-a)
  pab <- stats::pnorm(b) - stats::pnorm(a)
  cp <- cons*pab
  if (a<=0 & b>=0) {rate <- cp}
  if (a>0) {rate <- cp*exp(a^2/2)}
  return(rate)
}

#' Distribution function of truncated univariate normal distribution
#'
#' \code{ptuvn} calculates the cumulative distribution function (cdf) of truncated univariate normal distribution.
#'
#' @param x value at which cdf is desired.
#' @param mean mean of the underlying univariate normal distribution.
#' @param sd standard deviation of the underlying univariate normal distribution.
#' @param lower lower bound for truncation.
#' @param upper upper bound for truncation.
#'
#' @export
#' 
#' @return \code{ptuvn} returns the cumulative distribution function (with same dimension and type as \code{x}) of truncated univariate normal distribution.
#'
#' @examples
#' ptuvn(x= -3:3, mean=0, sd=1 ,lower= -2, upper=2)
#'
ptuvn <- function(x, mean, sd, lower, upper){
  result <- (stats::pnorm(x,mean,sd)- stats::pnorm(lower,mean,sd))/(stats::pnorm(upper,mean,sd) - stats::pnorm(lower,mean,sd))
  result[x <= lower] <- 0 ; result[x >= upper] <- 1

  result
}

#' Random number generation for truncated univariate normal distribution
#'
#' \code{rtuvn} simulates truncated univariate normal distribution within the interval.
#'
#' @param n number of random samples desired (sample size).
#' @param mean mean of the underlying univariate normal distribution.
#' @param sd standard deviation of the underlying univariate normal distribution.
#' @param lower lower bound for truncation.
#' @param upper upper bound for truncation.
#'
#' @export
#' 
#' @return \code{rtuvn} returns a vector of random number follows truncated univariate normal distribution.
#'
#' @examples
#' set.seed(1203)
#' ans <- rtuvn(n=1000, mean=1, sd=2, lower=-2, upper=3)
#' summary(ans)
#'
#' # Check if the sample matches with CDF by KS test
#' ks.test(ans,"ptuvn",1,2,-2,3)
#'
rtuvn <- function(n=1, mean=0, sd=1, lower, upper){
  # transform the boundaries
  a <- (lower - mean)/sd
  b <- (upper - mean)/sd

  # generate n samples from TN(0,1;a,b)
  Z <- rep(0,n)
  for (i in 1:n){
    temp <- imp(a,b)
    Z[i] <- temp$x
  }

  # transform the data back
  samp <- sd*Z + mean

  return(samp)
}

#' Acceptance rate of truncated univariate normal distribution rejection sampling
#'
#' \code{imp_acc} calculates the acceptance rate of truncated univariate standardized normal distribution rejection sampling for the truncation interval (a,b).
#'
#' @param a lower bound for truncation.
#' @param b upper bound for truncation.
#'
#' @export
#' 
#' @examples
#' imp_acc(1,Inf) # Case 1: [a,infty)
#' imp_acc(-1,1) # Case 2: 0 in [a,b], a<0<b
#' imp_acc(1,2) # Case 3: [a,b], a>=0
#'
imp_acc <- function(a,b){
  lowerb1 <- function(a){
    lowerb1 <- sqrt(pi/2)*exp(a^2/2)+a
    return(lowerb1)
  }

  # lower bound of b for normal vs unif in 0 in [a,b]
  lowerb2 <- function(a){
    lowerb2 <- sqrt(2*pi)+a
    return(lowerb2)
  }


  # lower bound of b for exp vs unif in [a,b]>=0
  lowerb <- function(a){
    lambda <- ( a+sqrt(a^2+4) )/2
    lowerb <- a+ exp(1/2)/lambda * exp(  ( a^2-a*sqrt(a^2+4) )/4  )
    return(lowerb)
  }

  # Case 1: [a,infty)
  imp_acc_case1 <- function(a,b=Inf){
    if (a<0) {
      rate <- norm_acc(a=a,b=b)
    } else {
      if (a<0.25696) {
        rate <- halfnorm_acc(a=a,b=b)
      } else {
        rate <- exp_acc_opt(a=a,b=b)
      }
    }
    return(rate)
  }

  # Case 2: 0 in [a,b], a<0<b
  imp_acc_case2 <- function(a,b){
    if (b>lowerb2(a)) {
      rate <- norm_acc(a=a,b=b)
    } else {
      rate <- unif_acc(a=a,b=b)}
    return(rate)
  }

  # Case 3: [a,b], a>=0
  imp_acc_case3 <- function(a,b){
    if (a<=0.25696) {
      blower1 <- lowerb1(a)
      if (b<= blower1) {
        rate <- unif_acc(a=a,b=b)
      } else {
        rate <- halfnorm_acc(a=a,b=b)
      }
    } else {
      blower2 <- lowerb(a)
      if (b<=blower2) {
        rate <- unif_acc(a=a,b=b)
      } else {
        rate <- exp_acc_opt(a=a,b=b)
      }
    }
    return(rate)
  }

  if (a==-Inf | b==Inf) {
    if (b==Inf) {
      rate <- imp_acc_case1(a=a)} else {
        rate <- imp_acc_case1(a=-b) }
  } else {
    if (a<0 & b>0) {rate <- imp_acc_case2(a=a,b=b)}
    if (a>=0) {rate <- imp_acc_case3(a=a,b=b)}
    if (b<=0) {rate <- imp_acc_case3(a=-b,b=-a)}
  }
  return(rate)
}

#' Translated-exponential rejection sampling
#'
#' \code{exp_rej} is used for translated-exponential rejection sampling.
#'
#' @param a lower bound
#' @param b upper bound
#' @param lam lambda for translated-exponential only
#'
#' @export
#' @return \code{exp_rej} returns a list
#' \code{x}: sampled value; and
#' \code{acc}: total number of draw used.
#'
#' @examples
#' set.seed(1)
#' exp_rej(a=1, b=Inf)
#'
exp_rej <- function(a,b=Inf,lam='default'){

  if (lam=='opt'){lambda <- ( a+sqrt(a^2+4) )/2} else {lambda=a}
  acc <- 0
  # rho is continuous at lambda=a, so don't need to change the expression of rho
  repeat{
    x <- stats::rweibull(1,shape=1,scale=1/lambda)+a
    u <- stats::runif(1)
    rho <- exp(-(x-lambda)^2/2)
    acc <- acc+1 # accept x at acc-th draw
    if (u <= rho & x<b)
      return(list(x=x,acc=acc))
  }
}

#' Random number generation for truncated multivariate normal distribution subject to linear inequality constraints
#'
#' \code{rtmvn} simulates truncated multivariate (p-dimensional) normal distribution subject to linear inequality constraints. The constraints should be written as a matrix (\code{D}) with \code{lower} and \code{upper} as the lower and upper bounds for those constraints respectively. Note that \code{D} can be non-full rank, which generalize many traditional methods.
#'
#' @param n number of random samples desired (sample size).
#' @param Mean mean vector of the underlying multivariate normal distribution.
#' @param Sigma positive definite covariance matrix of the underlying multivariate normal distribution.
#' @param D matrix or vector of coefficients of linear inequality constraints.
#' @param lower vector of lower bounds for truncation.
#' @param upper vector of upper bounds for truncation.
#' @param int initial value vector for Gibbs sampler (satisfying truncation), if \code{NULL} then determine automatically.
#' @param burn burn-in iterations discarded (default as \code{10}).
#' @param thin thinning lag (default as \code{1}).
#'
#' @export
#' 
#' @return \code{rtmvn} returns a (\code{n*p}) matrix (or vector when \code{n=1}) containing random numbers which approximately follows truncated multivariate  normal distribution.
#'
#' @examples
#' # Example for full rank with strong dependence
#' d <- 3
#' rho <- 0.9
#' Sigma <- matrix(0, nrow=d, ncol=d)
#' Sigma <- rho^abs(row(Sigma) - col(Sigma))
#'
#' D1 <- diag(1,d) # Full rank
#'
#' set.seed(1203)
#' ans.1 <- rtmvn(n=1000, Mean=1:d, Sigma, D=D1, lower=rep(-1,d), upper=rep(1,d),
#' int=rep(0,d), burn=50)
#'
#' apply(ans.1, 2, summary)
#'
#' # Example for non-full rank
#' d <- 3
#' rho <- 0.5
#' Sigma <- matrix(0, nrow=d, ncol=d)
#' Sigma <- rho^abs(row(Sigma) - col(Sigma))
#'
#' D2 <- matrix(c(1,1,1,0,1,0,1,0,1),ncol=d)
#' qr(D2)$rank # 2
#'
#' set.seed(1228)
#' ans.2 <- rtmvn(n=100, Mean=1:d, Sigma, D=D2, lower=rep(-1,d), upper=rep(1,d), burn=10)
#'
#' apply(ans.2, 2, summary)
#'
rtmvn <- function(n, Mean, Sigma, D=diag(1,length(Mean)), lower, upper, int=NULL, burn=10, thin=1){
  if (length(Mean) == 1) {
    result <- rtuvn(n=n, mean=Mean, sd=c(Sigma), lower=lower, upper=upper)
  } else{

  if ( any(lower >= upper)) stop("lower bound must be smaller than upper bound\n")

  bound.check <- 0

  if (!is.null(int)) {
    inits_test <- D%*%int
    lower.log <- inits_test >= lower + 1e-8  # small tol for get away from bound
    upper.log <- inits_test <= upper - 1e-8  # small tol for get away from bound
    bound.check <- prod(lower.log*upper.log)
    if(bound.check == 0) cat("initial is outside or too close from boundary, will be auto-corrected by ginv()!\n")
  } else if (bound.check == 0) {

    D.inv <- MASS::ginv(D)
    int <- D.inv%*%(lower + upper)/2
  }


  if( any (c(burn,thin,n) %% 1 != 0))  stop("burn, thin and n must be integer\n")
  if ( any(c(burn, thin, n -1) < 0) ) stop("burn, thin must be  non-negative interger, n must be positive integer\n")


  if(is.vector(D)==TRUE){
    Rtilde <- t(as.matrix(D))
    lower <- as.vector(lower)
    upper <- as.vector(upper)
  } else {
    Rtilde <- D
  }

  a <- lower - Rtilde%*%Mean
  b <- upper - Rtilde%*%Mean
  Sigma.chol <- t(chol(Sigma))
  R <- Rtilde%*%Sigma.chol

  p <- ncol(R) # number of parameters, i.e. length of beta vector
  #   m <- nrow(R) # number of constraints

  z <- solve(Sigma.chol)%*%(int-Mean)  # int is the initial value for the original problem

  keep.x <- matrix(NA, ncol=p, nrow=(thin+1)*n+burn)

  for (i in 1:((thin+1)*n+burn)){
    for (j in 1:p){  # p is the number of the degree of the multinormal
      rj <- as.vector(R[,j])   # the jth column of R
      Rj <- as.matrix(R[,-j])  # m by p-1 matrix by removing the jth column of R
      zj <- as.vector(z[-j])   # p-1 vector by removing the jth element
      a.temp <- a - Rj%*%zj
      b.temp <- b - Rj%*%zj

      pos <- rj>0   # if r_jk>0
      neg <- rj<0   # if r_jk<0

      if(sum(pos)==0){
        lower.pos <- -Inf
        upper.pos <- Inf
      } else {
        lower.pos <- max(a.temp[pos]/rj[pos])  # when r_jk>0, a associated with lower bound
        upper.pos <- min(b.temp[pos]/rj[pos])  #              b                 upper
      }

      if(sum(neg)==0){
        upper.neg <- Inf
        lower.neg <- -Inf
      } else {
        upper.neg <- min(a.temp[neg]/rj[neg])  # when r_jk<0, a                 upper
        lower.neg <- max(b.temp[neg]/rj[neg])  #              b                 lower
      }

      lower.j <- max(lower.pos,lower.neg)   # lower for z_j
      upper.j <- min(upper.pos,upper.neg)   # upper

      z[j] <- rtuvn(lower=lower.j,upper=upper.j)
    }
    ############################################################

    # for the original results
    x <- Sigma.chol%*%z+Mean
    keep.x[i,] <- x
  }

  final.ind <- 1:((thin+1)*n+burn)
  final.ind <- final.ind[(burn+1):length(final.ind)]
  final.ind <- seq(1,length(final.ind),by=thin+1) + thin + burn

  if (n == 1) {result <- c(keep.x[final.ind,])  } else{ result <- keep.x[final.ind,]}
  }

  return( result )
}

#' Uniform rejection sampling
#'
#' \code{unif_rej} is used for uniform rejection sampling.
#'
#' @param a lower bound
#' @param b upper bound
#'
#' @export
#' 
#' @return \code{unif_rej} returns a list
#' \code{x}: sampled value; and
#' \code{acc}: total number of draw used.
#'
#' @examples
#' set.seed(1)
#' unif_rej(a=1, b=2)
#'
unif_rej <- function(a,b){
  acc <- 0
  repeat{
    x <- stats::runif(1,a,b)
    u <- stats::runif(1)
    # different cases for the ratio
    if (0>=a & 0<=b){rho <- exp(-x^2/2)}
    if (a>0){rho <- exp(-(x^2-a^2)/2)}
    if (b<0){rho <- exp(-(x^2-b^2)/2)}
    acc <- acc+1 # accept x at acc-th draw
    if (u <= rho)
      return(list(x=x,acc=acc))
  }
}

#' Acceptance rate of half-normal rejection sampling
#'
#' \code{halfnorm_acc} calculates the acceptance rate of half-normal rejection sampling for the truncation interval (a,b).
#'
#' @param a lower bound for truncation.
#' @param b upper bound for truncation.
#'
#' @export
#' 
#' @examples
#' set.seed(1203)
#' halfnorm_acc(1,2)
#'
halfnorm_acc <- function(a,b){
  rate <- 2*(stats::pnorm(b) - stats::pnorm(a))
  return(rate)
}

#' Half-normal rejection sampling
#'
#' \code{halfnorm_rej} is used for half-normal rejection sampling.
#'
#' @param a lower bound
#' @param b upper bound
#'
#' @export
#' 
#' @return \code{halfnorm_rej} returns a list
#' \code{x}: sampled value; and
#' \code{acc}: total number of draw used.
#'
#' @examples
#' set.seed(1)
#' halfnorm_rej(a=1, b=Inf)
#'
halfnorm_rej <- function(a,b){
  acc <- 0
  repeat{
    x <- stats::rnorm(1)
    acc <- acc+1 # accept x at acc-th draw
    if (abs(x)>=a & abs(x)<=b)
      return(list(x=abs(x),acc=acc))
  }
}

#' Acceptance rate of normal rejection sampling
#'
#' \code{norm_acc} calculates the acceptance rate of normal rejection sampling for the truncation interval (a,b).
#'
#' @param a lower bound for truncation.
#' @param b upper bound for truncation.
#'
#' @export
#' 
#' @examples
#' set.seed(1203)
#' norm_acc(1,2)
#'
norm_acc <- function(a,b){
  rate <- stats::pnorm(b) - stats::pnorm(a)
  return(rate)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unif_acc.R
\name{unif_acc}
\alias{unif_acc}
\title{Acceptance rate of uniform rejection sampling}
\usage{
unif_acc(a, b)
}
\arguments{
\item{a}{lower bound for truncation.}

\item{b}{upper bound for truncation.}
}
\description{
\code{unif_acc} calculates the acceptance rate of uniform rejection sampling for the truncation interval (a,b).
}
\examples{
set.seed(1203)
unif_acc(1,2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imp_acc.R
\name{imp_acc}
\alias{imp_acc}
\title{Acceptance rate of truncated univariate normal distribution rejection sampling}
\usage{
imp_acc(a, b)
}
\arguments{
\item{a}{lower bound for truncation.}

\item{b}{upper bound for truncation.}
}
\description{
\code{imp_acc} calculates the acceptance rate of truncated univariate standardized normal distribution rejection sampling for the truncation interval (a,b).
}
\examples{
imp_acc(1,Inf) # Case 1: [a,infty)
imp_acc(-1,1) # Case 2: 0 in [a,b], a<0<b
imp_acc(1,2) # Case 3: [a,b], a>=0

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtmvn.R
\name{rtmvn}
\alias{rtmvn}
\title{Random number generation for truncated multivariate normal distribution subject to linear inequality constraints}
\usage{
rtmvn(
  n,
  Mean,
  Sigma,
  D = diag(1, length(Mean)),
  lower,
  upper,
  int = NULL,
  burn = 10,
  thin = 1
)
}
\arguments{
\item{n}{number of random samples desired (sample size).}

\item{Mean}{mean vector of the underlying multivariate normal distribution.}

\item{Sigma}{positive definite covariance matrix of the underlying multivariate normal distribution.}

\item{D}{matrix or vector of coefficients of linear inequality constraints.}

\item{lower}{vector of lower bounds for truncation.}

\item{upper}{vector of upper bounds for truncation.}

\item{int}{initial value vector for Gibbs sampler (satisfying truncation), if \code{NULL} then determine automatically.}

\item{burn}{burn-in iterations discarded (default as \code{10}).}

\item{thin}{thinning lag (default as \code{1}).}
}
\value{
\code{rtmvn} returns a (\code{n*p}) matrix (or vector when \code{n=1}) containing random numbers which approximately follows truncated multivariate  normal distribution.
}
\description{
\code{rtmvn} simulates truncated multivariate (p-dimensional) normal distribution subject to linear inequality constraints. The constraints should be written as a matrix (\code{D}) with \code{lower} and \code{upper} as the lower and upper bounds for those constraints respectively. Note that \code{D} can be non-full rank, which generalize many traditional methods.
}
\examples{
# Example for full rank with strong dependence
d <- 3
rho <- 0.9
Sigma <- matrix(0, nrow=d, ncol=d)
Sigma <- rho^abs(row(Sigma) - col(Sigma))

D1 <- diag(1,d) # Full rank

set.seed(1203)
ans.1 <- rtmvn(n=1000, Mean=1:d, Sigma, D=D1, lower=rep(-1,d), upper=rep(1,d),
int=rep(0,d), burn=50)

apply(ans.1, 2, summary)

# Example for non-full rank
d <- 3
rho <- 0.5
Sigma <- matrix(0, nrow=d, ncol=d)
Sigma <- rho^abs(row(Sigma) - col(Sigma))

D2 <- matrix(c(1,1,1,0,1,0,1,0,1),ncol=d)
qr(D2)$rank # 2

set.seed(1228)
ans.2 <- rtmvn(n=100, Mean=1:d, Sigma, D=D2, lower=rep(-1,d), upper=rep(1,d), burn=10)

apply(ans.2, 2, summary)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtmvt.R
\name{rtmvt}
\alias{rtmvt}
\title{Random number generation for truncated multivariate Student's t distribution subject to linear inequality constraints}
\usage{
rtmvt(n, Mean, Sigma, nu, D, lower, upper, int = NULL, burn = 10, thin = 1)
}
\arguments{
\item{n}{number of random samples desired (sample size).}

\item{Mean}{location vector of the multivariate Student's t distribution.}

\item{Sigma}{positive definite dispersion matrix of the multivariate t distribution.}

\item{nu}{degrees of freedom for Student-t distribution.}

\item{D}{matrix or vector of coefficients of linear inequality constraints.}

\item{lower}{lower bound vector for truncation.}

\item{upper}{upper bound vector for truncation.}

\item{int}{initial value vector for Gibbs sampler (satisfying truncation), if \code{NULL} then determine automatically.}

\item{burn}{burn-in iterations discarded (default as \code{10}).}

\item{thin}{thinning lag (default as \code{1}).}
}
\value{
\code{rtmvt} returns a (\code{n*p}) matrix (or vector when \code{n=1}) containing random numbers which follows truncated multivariate Student-t distribution.
}
\description{
\code{rtmvt} simulates truncated multivariate (p-dimensional) Student's t distribution subject to linear inequality constraints. The constraints should be written as a matrix (\code{D}) with \code{lower} and \code{upper} as the lower and upper bounds for those constraints respectively. Note that \code{D} can be non-full rank, which generalizes many traditional methods.
}
\examples{
# Example for full rank
d <- 3
rho <- 0.5
nu <- 10
Sigma <- matrix(0, nrow=d, ncol=d)
Sigma <- rho^abs(row(Sigma) - col(Sigma))

D1 <- diag(1,d) # Full rank

set.seed(1203)
ans.t <- rtmvt(n=1000, Mean=1:d, Sigma, nu=nu, D=D1, lower=rep(-1,d), upper=rep(1,d),
burn=50, thin=0)

apply(ans.t, 2, summary)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unif_rejection.R
\name{unif_rej}
\alias{unif_rej}
\title{Uniform rejection sampling}
\usage{
unif_rej(a, b)
}
\arguments{
\item{a}{lower bound}

\item{b}{upper bound}
}
\value{
\code{unif_rej} returns a list
\code{x}: sampled value; and
\code{acc}: total number of draw used.
}
\description{
\code{unif_rej} is used for uniform rejection sampling.
}
\examples{
set.seed(1)
unif_rej(a=1, b=2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normal_acc.R
\name{norm_acc}
\alias{norm_acc}
\title{Acceptance rate of normal rejection sampling}
\usage{
norm_acc(a, b)
}
\arguments{
\item{a}{lower bound for truncation.}

\item{b}{upper bound for truncation.}
}
\description{
\code{norm_acc} calculates the acceptance rate of normal rejection sampling for the truncation interval (a,b).
}
\examples{
set.seed(1203)
norm_acc(1,2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ptuvn.R
\name{ptuvn}
\alias{ptuvn}
\title{Distribution function of truncated univariate normal distribution}
\usage{
ptuvn(x, mean, sd, lower, upper)
}
\arguments{
\item{x}{value at which cdf is desired.}

\item{mean}{mean of the underlying univariate normal distribution.}

\item{sd}{standard deviation of the underlying univariate normal distribution.}

\item{lower}{lower bound for truncation.}

\item{upper}{upper bound for truncation.}
}
\value{
\code{ptuvn} returns the cumulative distribution function (with same dimension and type as \code{x}) of truncated univariate normal distribution.
}
\description{
\code{ptuvn} calculates the cumulative distribution function (cdf) of truncated univariate normal distribution.
}
\examples{
ptuvn(x= -3:3, mean=0, sd=1 ,lower= -2, upper=2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exp_rejection.R
\name{exp_rej}
\alias{exp_rej}
\title{Translated-exponential rejection sampling}
\usage{
exp_rej(a, b = Inf, lam = "default")
}
\arguments{
\item{a}{lower bound}

\item{b}{upper bound}

\item{lam}{lambda for translated-exponential only}
}
\value{
\code{exp_rej} returns a list
\code{x}: sampled value; and
\code{acc}: total number of draw used.
}
\description{
\code{exp_rej} is used for translated-exponential rejection sampling.
}
\examples{
set.seed(1)
exp_rej(a=1, b=Inf)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exp_acc.R
\name{exp_acc_opt}
\alias{exp_acc_opt}
\title{Acceptance rate of translated-exponential rejection sampling}
\usage{
exp_acc_opt(a, b)
}
\arguments{
\item{a}{lower bound for truncation.}

\item{b}{upper bound for truncation.}
}
\description{
\code{exp_acc_opt} calculates the acceptance rate of translated-exponential rejection sampling for the truncation interval (a,b).
}
\examples{

set.seed(1203)
exp_acc_opt(1,2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imp.R
\name{imp}
\alias{imp}
\title{Rejection sampling of standardized truncated univariate normal distribution}
\usage{
imp(a, b)
}
\arguments{
\item{a}{lower bound for truncation.}

\item{b}{upper bound for truncation (must be \code{> a}).}
}
\value{
\code{imp} returns a list
\code{x}: sampled value; and
\code{acc}: total number of draw used.
}
\description{
\code{imp} contains a general function for rejection sampling of standardized truncated univariate normal distribution in (a,b).
}
\examples{
imp(1,Inf) # Case 1: [a,infty)
imp(-1,1) # Case 2: 0 in [a,b], a<0<b
imp(1,2) # Case 3: [a,b], a>=0

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtuvn.R
\name{rtuvn}
\alias{rtuvn}
\title{Random number generation for truncated univariate normal distribution}
\usage{
rtuvn(n = 1, mean = 0, sd = 1, lower, upper)
}
\arguments{
\item{n}{number of random samples desired (sample size).}

\item{mean}{mean of the underlying univariate normal distribution.}

\item{sd}{standard deviation of the underlying univariate normal distribution.}

\item{lower}{lower bound for truncation.}

\item{upper}{upper bound for truncation.}
}
\value{
\code{rtuvn} returns a vector of random number follows truncated univariate normal distribution.
}
\description{
\code{rtuvn} simulates truncated univariate normal distribution within the interval.
}
\examples{
set.seed(1203)
ans <- rtuvn(n=1000, mean=1, sd=2, lower=-2, upper=3)
summary(ans)

# Check if the sample matches with CDF by KS test
ks.test(ans,"ptuvn",1,2,-2,3)

}
